﻿Imports System.Runtime.CompilerServices

Public Module Extensions


   Public Structure ApproachResult
      Public ReadOnly Node As TreeNode
      Public ReadOnly StepsDone, InsertIndex As Integer
      Public ReadOnly Segments As IList(Of String)

      Public Sub New( _
            ByVal Node As TreeNode, _
            ByVal Segments As IList(Of String), _
            ByVal StepsDone As Integer, InsertIndex As Integer)
         Me.Node = Node
         Me.Segments = Segments
         Me.StepsDone = StepsDone
         Me.InsertIndex = InsertIndex
      End Sub

      Public ReadOnly Property IsFullMatch() As Boolean
         Get
            Return Segments.Count = StepsDone
         End Get
      End Property
   End Structure 'ApproachResult

   ''' <summary>
   ''' sucht in Nodes und SubNodes anhand eines Pfades nach dem bezeichneten Node.
   ''' Bei Nichtexistenz des ZielNode den übergeordneten, in den der ZielNode zu erstellen wäre
   ''' </summary>
   ''' <param name="PathSegments">Die Segmente des Pfades</param>
   ''' <returns>ein 
   ''' <seealso cref="ApproachResult"> ApproachResult, 
   ''' das den gefundenen Node enthält, und die Anzahl abgearbeiteter Segmente
   ''' </seealso>
   ''' </returns>
   <Extension()> _
   Public Function Approach(ByVal Nodes As TreeNodeCollection, _
         ByVal PathSegments As IList(Of String)) As ApproachResult
      Dim Nd As TreeNode = Nothing, I As Integer
      For I = 0 To PathSegments.Count - 1
         Dim indx = Nodes.FindIndexByText(PathSegments(I))
         If indx < 0 Then Return New ApproachResult(Nd, PathSegments, I, Not indx)
         Nd = Nodes(indx)
         Nodes = Nd.Nodes
      Next
      Return New ApproachResult(Nd, PathSegments, I, -1)
   End Function

   <Extension()> _
   Public Function FindIndexByText(nodes As TreeNodeCollection, txt As String) As Integer
      For i = 0 To nodes.Count - 1
         Select Case String.Compare(nodes(i).Text, txt, True)
            Case Is > 0 : Return Not i
            Case 0 : Return i
            Case Else
         End Select
      Next
      Return Not nodes.Count
   End Function

   <Extension()> _
   Public Function Approach(ByVal TV As TreeView, ByVal NodePath As String) As ApproachResult
      Return TV.Nodes.Approach(NodePath.Split(TV.PathSeparator(0)))
   End Function

End Module
